//  Stack-based calculator

//  Uses reverse polish (postfix) notation and a stack to implement a basic
//  calculator.  The user would enter 1 2 3 * + = to get 2*3 + 1 =
//  This would result in the following sequence:
//      push 1, push 2, push 3
//      (pop2) * (pop3), push 6
//      (pop6) + (pop1), push 7
//      (pop7) and print

#include <iostream>

using namespace std;

const int STACKSIZE = 20;
int stack[ STACKSIZE ], stackPointer = 0;

void push(int n)
{
    if (stackPointer <= STACKSIZE-1)
	stack[stackPointer++] = n;
}

int pop()
{
    if (stackPointer > 0)
	stackPointer--;
    return( stack[stackPointer] );
}

//  The main() program gets keyboard input and does the math...

int power(int base, int pow);   // function prototype...

int main()
{
    char ch;
    int second;
    
    cin >> ch;
    while (ch != '=') {
	if (ch >= '0' && ch <= '9') {
	    cin.putback(ch);
	    cin >> second;
	    push(second);
	} else {
	    switch (ch) {
	    case '+' :  push(pop() + pop()); break;
	    case '*' :  push(pop() * pop()); break;
	    case '-' :  push(pop() - pop()); break;
	    case '/' :
		second = pop();
		if (second == 0) {
		    cerr << "Divide by 0." << endl;
		    return 1;
		}
		push(pop() / second);
		break;
	    case '^' :
		second = pop();
		if (second < 0) {
		    cerr << "Raise to negative power." << endl;
		    return 1;
		}
		push(power(pop(), second)); break;
	    default :
		cerr << "Input error." << endl;
		return 2;
	    }
	}
	cin >> ch;
    }
    cout << pop() << endl;
    return 0;
}

int power(int base, int pow)
{
    int orig = base;
    
    if (pow == 0)
	base = 1;
    while (pow-- > 1)
	base *= orig;
    return( base );
}
